﻿/*

	QRCode-MATRIX
	Luigi De Aloisio 2013
	www.nostatic.it



	Simple Version QRCode + 2 Images 
	........................................................
	This script require a reference folder containing:
		white image named "white.png"
		black image named "black.png"
		qrcode image named "qrcode.png" (31 cells resolution)
	........................................................

	open in ExtendScript Toolkit.app with target Photoshop
	or
	in Photoshop choose File > Scripts > Browse and navigate to the script.
	

*/


#target photoshop
// Make Photoshop the frontmost application
app.bringToFront();

// Save the current preferences
var startRulerUnits 	    = app.preferences.rulerUnits;
var startTypeUnits          = app.preferences.typeUnits;
var startDisplayDialogs     = app.displayDialogs;

// Set Adobe Photoshop to use pixels and display no dialogs 
app.preferences.rulerUnits  = Units.PIXELS;
app.preferences.typeUnits   = TypeUnits.PIXELS;
app.displayDialogs          = DialogModes.NO;
 

//--------------------------------------------------------------------------------------------
//  init variables
//--------------------------------------------------------------------------------------------
var doc,library_white,library_black,qrcode,whitelayer,blacklayer,picker;
var myAction={};
//--------------------------------------------------------------------------------------------
// setup parameters
//--------------------------------------------------------------------------------------------
myAction.DOCUMENTOUTWIDTH=2000; //final image in pixel
myAction.DOCUMENTOUTRESOLUTION = 72; //DPI
myAction.QRCODECELLS	= 31;
myAction.IMAGEFORMAT	= "jpg"; // input image format

myAction.CELLOUTSIZE	= (myAction.DOCUMENTOUTWIDTH/myAction.QRCODECELLS);
myAction.DOCUMENTOUTSIZE = Math.floor(myAction.CELLOUTSIZE*myAction.QRCODECELLS);
//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
myAction.SetupQRcode=function (){
	var inputFolder = Folder.selectDialog("Select a reference folder" )
	if (inputFolder != null) {

    // close all documents
    while (app.documents.length) {app.activeDocument.close(SaveOptions.DONOTSAVECHANGES);}
     
    // Create final image
	doc = app.documents.add(this.DOCUMENTOUTSIZE, this.DOCUMENTOUTSIZE, myAction.DOCUMENTOUTRESOLUTION, "Nostatic_QRcoded_Matrix", NewDocumentMode.RGB, DocumentFill.BACKGROUNDCOLOR);
	
	// load dark image reference
	library_black=loadImage( File(inputFolder+"/black."+myAction.IMAGEFORMAT));
    library_black.selection.selectAll();
    library_black.selection.copy()
    library_black.close();

    // add dark temp layer  
	library_black = doc.artLayers.add();  
	library_black.name = "library_black"; 
	doc.paste();
	setScale(library_black,this.CELLOUTSIZE,this.CELLOUTSIZE);
	setPosition(library_black,0,0);
     
    // load light image reference 
    library_white=loadImage(File(inputFolder+"/white."+myAction.IMAGEFORMAT));
    library_white.selection.selectAll();
    library_white.selection.copy()
    library_white.close();
    
     // add light temp layer  
    library_white = doc.artLayers.add();  
    library_white.name = "library_white"; 
    doc.paste();
    setScale(library_white,this.CELLOUTSIZE,this.CELLOUTSIZE);
    setPosition(library_white,0,0);
    
    // add output layers 
    whitelayer =doc.artLayers.add();  
    whitelayer.name = "white"; 
    blacklayer = doc.artLayers.add();  
    blacklayer.name = "black"; 

    // load qrcode reference
    var tmpqrcode = loadImage( File(inputFolder+"/qrcode.png"));
    tmpqrcode.selection.selectAll();
    tmpqrcode.selection.copy();
    tmpqrcode.close();

    // add qrcode layer on the top
    qrcode = doc.artLayers.add();  
    qrcode.name = "qrcode"; 
    doc.paste();
    setPosition(qrcode,0,0);

	// add picker color
    doc.colorSamplers.removeAll();
    picker = doc.colorSamplers.add([0,0]); 
	
	this.ProcessQRcode();
	
	// free memory
	qrcode.remove();
    library_white.remove();
    library_black.remove();

	}else{
      alert ("action stopped!", "error", 1);  
	}
}

myAction.ProcessQRcode=function(){
	var x,y;
    var size=qrcode.bounds[2]/this.QRCODECELLS;
    var center=size/2;
    var count=0;
    var flag=false;
    var libreria,livello,clone,colore;
	
	for (var i = 0; i < this.QRCODECELLS; i++) {
        for (var j = 0; j < this.QRCODECELLS; j++) {
        
            x=Math.floor(j*size)+center;
            y=Math.floor(i*size)+center;
           
            // legge colore  
            colore=getColour(x,y);
            
            flag=colore>10;
            
            // seleziona libreria
            libreria=(flag)?library_white:library_black
            
			// seleziona livello destinazione
			livello=(flag)?whitelayer:blacklayer;
			clone = libreria.duplicate(livello,ElementPlacement.PLACEBEFORE)
    
            setPosition(clone,j*this.CELLOUTSIZE,i*this.CELLOUTSIZE);
            clone.merge();
               
			// clear cache
			count++;
             if (count>=20) {
				app.purge(PurgeTarget.ALLCACHES);
                count=0;
			}
        }
    }
} 

myAction.InitQRCode=function (){
	$.writeln ("start action");
	app.togglePalettes();

	this.SetupQRcode();
	
	app.togglePalettes();
	$.writeln ("end action");
}

myAction.InitQRCode();

//--------------------------------------------------------------------------------------------
// Utility Functions
//--------------------------------------------------------------------------------------------
function getColour(x,y){
	picker.move([x,y]);
    return picker.color.rgb.red;
}
function loadImage(path){
	
	try {
	var img = open(path);
	return img;     
	} catch(e) {
	alert(e);
	return null;
	}
}
function setSelection(x,y,lar,alt){
	var sel = Array(Array(x, y),Array(x+lar, y),Array(x+lar, y+alt), Array(x, y+alt),Array(x, y));
	return sel;
}
function setPosition( lyr, x, y ){// layerObject, Number, Number
     // if can not move layer return
     if(lyr.isBackgroundLayer||lyr.positionLocked) return
	 
     // get the layer bounds
     var layerBounds = lyr.bounds;
      // get top left position
     var layerX = layerBounds[0].value;
     var layerY = layerBounds[1].value;
     // the difference between where layer needs to be and is now
     var deltaX = x-layerX;
     var deltaY = y-layerY;
     // move the layer into position
     lyr.translate (deltaX, deltaY);
}
function setScale(layer,Width , Height){
	
	if(layer.isBackgroundLayer ||layer.positionLocked) return;
	
	var LB = layer.bounds;
	var sx =100/(LB[2].value - LB[0].value);
	var sy =100/(LB[3].value - LB[1].value);
	var NewWidth = sx * Width;
	var NewHeight = sy * Height;
	layer.resize(Number(NewWidth),Number(NewHeight),AnchorPosition.TOPLEFT);
}






//--------------------------------------------------------------------------------------------
// Reset the application preferences 
//--------------------------------------------------------------------------------------------
app.preferences.rulerUnits = startRulerUnits 
app.preferences.typeUnits = startTypeUnits
app.displayDialogs = startDisplayDialogs
